@@ -6,81 +6,38 @@ import os
 import re
 import logging
 import shutil
-import json
 import sys
 import fosslight_util.constant as constant
+import urllib.request
 from yaml import safe_dump
 from fosslight_util.set_log import init_log
-from fosslight_util.spdx_licenses import get_spdx_licenses_json
+from fosslight_util.spdx_licenses import get_spdx_licenses_json, get_license_from_nick
 from fosslight_util.parsing_yaml import find_sbom_yaml_files, parsing_yml
 from fosslight_util.output_format import check_output_format
 from datetime import datetime
-from fosslight_prechecker._precheck import precheck_for_project, precheck_for_files, dump_error_msg, get_path_to_find
+from fosslight_prechecker._precheck import precheck_for_project, precheck_for_files, dump_error_msg, \
+                                           get_path_to_find, DEFAULT_EXCLUDE_EXTENSION_FILES
+from fosslight_prechecker._result import get_total_file_list
 from reuse.header import run as reuse_header
 from reuse.download import run as reuse_download
 from reuse._comment import EXTENSION_COMMENT_STYLE_MAP_LOWERCASE
 from reuse._main import parser as reuse_arg_parser
 from reuse.project import Project
+from bs4 import BeautifulSoup
+from pathlib import Path
 
 
 PKG_NAME = "fosslight_prechecker"
 LICENSE_INCLUDE_FILES = ["license", "license.md", "license.txt", "notice"]
 EXCLUDE_DIR = ["test", "tests", "doc", "docs"]
 EXCLUDE_PREFIX = ("test", ".", "doc", "__")
-RESOURCES_DIR = 'resources'
-LICENSES_JSON_FILE = 'convert_license.json'
+OPENSOURCE_LGE_COM_URL_PREFIX = "https://opensource.lge.com/license/"
 _result_log = {}
 spdx_licenses = []
 
 logger = logging.getLogger(constant.LOGGER_NAME)
 
 
-def get_licenses_from_json():
-    licenses = {}
-    licenses_file = os.path.join(RESOURCES_DIR, LICENSES_JSON_FILE)
-
-    try:
-        base_dir = sys._MEIPASS
-    except Exception:
-        base_dir = os.path.dirname(__file__)
-
-    file_withpath = os.path.join(base_dir, licenses_file)
-    try:
-        with open(file_withpath, 'r') as f:
-            licenses = json.load(f)
-    except Exception as ex:
-        dump_error_msg(f"Error to get license from json file : {ex}")
-
-    return licenses
-
-
-def check_file_extension(file_list):
-    if file_list != "":
-        for file in file_list:
-            try:
-                file_extension = os.path.splitext(file)[1].lower()
-                if file_extension == "":
-                    logger.info(f" No extension file(s) : {file}")
-                if file_extension in EXTENSION_COMMENT_STYLE_MAP_LOWERCASE:
-                    yield file
-            except Exception as ex:
-                dump_error_msg(f"Error - Unknown error to check file extension: {ex}")
-
-
-def check_license_and_copyright(path_to_find, all_files, missing_license, missing_copyright):
-    # Check file extension for each list
-    all_files_filtered = list(check_file_extension(all_files))
-    missing_license_filtered = list(check_file_extension(missing_license))
-    missing_copyright_filtered = list(check_file_extension(missing_copyright))
-
-    skip_files = sorted(set(all_files_filtered) - set(missing_license_filtered) - set(missing_copyright_filtered))
-    logger.info(f"\n# File list that have both license and copyright : {len(skip_files)} / {len(all_files_filtered)}")
-
-    precheck_for_files(path_to_find, skip_files)
-
-    return missing_license_filtered, missing_copyright_filtered
-
-
 def convert_to_spdx_style(input_string):
     input_string = input_string.replace(" ", "-")
     input_converted = f"LicenseRef-{input_string}"
@@ -88,19 +45,20 @@ def convert_to_spdx_style(input_string):
 
 
 def check_input_license_format(input_license):
-    if input_license in spdx_licenses:
-        return input_license
+    for spdx in spdx_licenses:
+        if input_license.casefold() == spdx.casefold():
+            return spdx
 
     if input_license.startswith('LicenseRef-'):
         return input_license
 
-    licensesfromJson = get_licenses_from_json()
+    licensesfromJson = get_license_from_nick()
     if licensesfromJson == "":
         dump_error_msg(" Error - Return Value to get license from Json is none")
 
     try:
         # Get frequetly used license from json file
-        converted_license = licensesfromJson.get(input_license.lower())
+        converted_license = licensesfromJson.get(input_license.casefold())
         if converted_license is None:
             converted_license = convert_to_spdx_style(input_license)
     except Exception as ex:
@@ -124,7 +82,7 @@ def input_license_while_running():
     input_license = ""
 
     logger.info("# Select a license to write in the license missing files ")
-    select = input("   1.MIT,  2.Apache-2.0,  3.LGE-Proprietary,  4.Manaully Input,  5.Not select now : ")
+    select = input("\t1.MIT\n \t2.Apache-2.0\n \t3.LGE-Proprietary\n \t4.Manual Input\n \t5.Not select now\n- Choose one from the list: ")
     if select == '1' or select == 'MIT':
         input_license = 'MIT'
     elif select == '2' or select == 'Apache-2.0':
@@ -140,7 +98,7 @@ def input_license_while_running():
 
 def input_copyright_while_running():
     input_copyright = ""
-    input_copyright = input("# Input Copyright to write in the copyright missing files (ex, <year> <name>) : ")
+    input_copyright = input("# Input Copyright to write in the copyright missing files (ex, <year> <name>): ")
     if input_copyright == 'Quit' or input_copyright == 'quit' or input_copyright == 'Q':
         return
 
@@ -222,12 +180,11 @@ def get_allfiles_list(path):
                     dirs.remove(dir)
                     continue
             for file in files:
-                file_lower_case = file.lower()
-                file_abs_path = os.path.join(root, file_lower_case)
+                file_abs_path = os.path.join(root, file)
                 file_rel_path = os.path.relpath(file_abs_path, path)
                 yield file_rel_path
     except Exception as ex:
-        dump_error_msg(f"Error_Get_AllFiles : {ex}")
+        dump_error_msg(f"Error - get all files list : {ex}")
 
 
 def save_result_log():
@@ -238,22 +195,67 @@ def save_result_log():
         logger.warning(f"Failed to print add result log. : {ex}")
 
 
-def copy_to_root(input_license):
+def copy_to_root(path_to_find, input_license):
     lic_file = f"{input_license}.txt"
     try:
-        source = os.path.join('LICENSES', f'{lic_file}')
-        destination = 'LICENSE'
+        source = os.path.join(path_to_find, 'LICENSES', f'{lic_file}')
+        destination = os.path.join(path_to_find, 'LICENSE')
         shutil.copyfile(source, destination)
     except Exception as ex:
         dump_error_msg(f"Error - Can't copy license file: {ex}")
 
 
+def lge_lic_download(path_to_find, input_license):
+    success = False
+
+    input_license_url = input_license.replace(' ', '_').replace('/', '_').replace('LicenseRef-', '').replace('-', '_')
+    lic_url = OPENSOURCE_LGE_COM_URL_PREFIX + input_license_url + ".html"
+
+    try:
+        html = urllib.request.urlopen(lic_url)
+        source = html.read()
+        html.close()
+    except urllib.error.URLError:
+        logger.error("Invalid URL address")
+    except ValueError as val_err:
+        logger.error(f"Invalid Value : {val_err}")
+    except Exception as ex:
+        logger.error(f"Error to open url - {lic_url} : {ex}")
+
+    soup = BeautifulSoup(source, 'html.parser')
+    try:
+        lic_text = soup.find("p", "bdTop")
+        Path(os.path.join(os.getcwd(), path_to_find, 'LICENSES')).mkdir(parents=True, exist_ok=True)
+        lic_file_path = os.path.join(path_to_find, 'LICENSES', f'{input_license}.txt')
+
+        with open(lic_file_path, 'w', encoding='utf-8') as f:
+            f.write(lic_text.get_text(separator='\n'))
+        if os.path.isfile(lic_file_path):
+            logger.info(f"Successfully downloaded {lic_file_path}")
+            success = True
+    except Exception as ex:
+        logger.error(f"Error to download license from LGE : {ex}")
+    return success
+
+
+def present_license_file(path_to_find, lic):
+    present = False
+    lic_file_path = os.path.join(os.getcwd(), path_to_find, 'LICENSES')
+    file_name = f"{lic}.txt"
+    if file_name in os.listdir(lic_file_path):
+        present = True
+    return present
+
+
 def find_representative_license(path_to_find, input_license):
     files = []
     found_file = []
     found_license_file = False
     main_parser = reuse_arg_parser()
     prj = Project(path_to_find)
+    reuse_return_code = 0
+    success_from_lge = False
+    present_lic = False
 
     all_items = os.listdir(path_to_find)
     for item in all_items:
@@ -262,28 +264,33 @@ def find_representative_license(path_to_find, input_license):
 
     for file in files:
         file_lower_case = file.lower()
-
         if file_lower_case in LICENSE_INCLUDE_FILES or file_lower_case.startswith("license") or file_lower_case.startswith("notice"):
             found_file.append(file)
             found_license_file = True
 
-    if found_license_file:
-        logger.warning(f" # Found representative license file : {found_file}")
-    else:
-        input_license = check_input_license_format(input_license)
+    input_license = check_input_license_format(input_license)
+    logger.info(f" - Input License : {input_license}")
 
-        logger.info(f" Input License : {input_license}")
-        parsed_args = main_parser.parse_args(['download', f"{input_license}"])
+    parsed_args = main_parser.parse_args(['download', f"{input_license}"])
 
-        try:
-            logger.warning(" # There is no representative license file")
-            reuse_download(parsed_args, prj)
-            copy_to_root(input_license)
-            if input_license in spdx_licenses:
-                logger.warning(f" # Created Representative License File : {input_license}.txt")
+    try:
+        # 0: successfully downloaded, 1: failed to download
+        reuse_return_code = reuse_download(parsed_args, prj)
+        # Check if the license text file is present
+        present_lic = present_license_file(path_to_find, input_license)
 
-        except Exception as ex:
-            dump_error_msg(f"Error - download representative license text: {ex}")
+        if reuse_return_code == 1 and not present_lic:
+            # True : successfully downloaded from LGE
+            success_from_lge = lge_lic_download(path_to_find, input_license)
+
+        if reuse_return_code == 0 or success_from_lge:
+            if found_license_file:
+                logger.info(f"# Found representative license file : {found_file}\n")
+            else:
+                logger.warning(f"# Created Representative License File : {input_license}.txt\n")
+                copy_to_root(path_to_find, input_license)
+    except Exception as ex:
+        dump_error_msg(f"Error - download representative license text: {ex}")
 
 
 def is_exclude_dir(dir_path):
@@ -316,7 +323,7 @@ def download_oss_info_license(base_path, input_license=""):
         logger.info(f"\n # There is OSS Package Info file(s) : {oss_yaml_files}\n")
 
     for oss_pkg_file in oss_yaml_files:
-        _, license_list = parsing_yml(oss_pkg_file, base_path)
+        _, license_list, _ = parsing_yml(oss_pkg_file, base_path)
 
     for lic in license_list:
         converted_lic_list.append(check_input_license_format(lic))
@@ -333,7 +340,7 @@ def download_oss_info_license(base_path, input_license=""):
 
 
 def add_content(target_path="", input_license="", input_copyright="", output_path="", need_log_file=True):
-    global _result_log
+    global _result_log, spdx_licenses
     _check_only_file_mode = False
     file_to_check_list = []
 
@@ -368,9 +375,6 @@ def add_content(target_path="", input_license="", input_copyright="", output_pat
     except Exception as ex:
         dump_error_msg(f"Error access to get_spdx_licenses_json : {ex}")
 
-    if input_license != "":
-        find_representative_license(path_to_find, input_license)
-
     # File Only mode (-f option)
     if _check_only_file_mode:
         main_parser = reuse_arg_parser()
@@ -415,22 +419,31 @@ def add_content(target_path="", input_license="", input_copyright="", output_pat
         # Download license text file of OSS-pkg-info.yaml
         download_oss_info_license(path_to_find, input_license)
 
-        # Get all files List in path
-        all_files_list = get_allfiles_list(path_to_find)
-
         # Get missing license / copyright file list
-        missing_license, missing_copyright, _, project, _ = precheck_for_project(path_to_find, need_log_file)
+        missing_license, missing_copyright, _, project, prj_report = precheck_for_project(path_to_find)
 
-        # Print Skipped Files
-        missing_license_filtered, missing_copyright_filtered = \
-            check_license_and_copyright(path_to_find, all_files_list, missing_license, missing_copyright)
+        # Get total files except excluded file
+        total_files_excluded = get_total_file_list(path_to_find, prj_report, DEFAULT_EXCLUDE_EXTENSION_FILES)
+        skip_files = sorted(set(total_files_excluded) - set(missing_license) - set(missing_copyright))
+        logger.info(f"\n# File list that have both license and copyright : {len(skip_files)} / {len(total_files_excluded)}")
+
+        # Filter by file extension
+        missing_license = [file for file in missing_license if os.path.splitext(file)[1].lower() in EXTENSION_COMMENT_STYLE_MAP_LOWERCASE]
+        missing_copyright = [file for file in missing_copyright if os.path.splitext(file)[1].lower() in EXTENSION_COMMENT_STYLE_MAP_LOWERCASE]
+
+        # Check license and copyright of each file
+        precheck_for_files(path_to_find, skip_files)
 
         # Set missing license and copyright
-        set_missing_license_copyright(missing_license_filtered,
-                                      missing_copyright_filtered,
+        set_missing_license_copyright(missing_license,
+                                      missing_copyright,
                                       project,
                                       path_to_find,
                                       input_license,
                                       input_copyright)
 
+    # Find and create representative license file
+    if input_license != "" and len(missing_license) > 0:
+        find_representative_license(path_to_find, input_license)
+
     save_result_log()
