@@ -5,25 +5,27 @@
 import os
 import io
 import sys
+import platform
 import yaml
 import fnmatch
 import xml.etree.ElementTree as ET
 import logging
 import fosslight_util.constant as constant
+import re
 from pathlib import Path
 from reuse.project import Project
 from fosslight_prechecker._result_html import result_for_html
 from fosslight_util.parsing_yaml import find_sbom_yaml_files, parsing_yml
 from fosslight_util.output_format import check_output_format
-import re
 
 
 CUSTOMIZED_FORMAT_FOR_PRECHECKER = {'html': '.html', 'xml': '.xml', 'yaml': '.yaml'}
-RULE_LINK = "https://oss.lge.com/guide/process/osc_process/1-identification/copyright_license_rule.html"
+RULE_LINK = "https://opensource.lge.com/guide/19"
 MSG_REFERENCE = "Ref. Copyright and License Writing Rules in Source Code. : " + RULE_LINK
 MSG_FOLLOW_LIC_TXT = "Follow the Copyright and License Writing Rules in Source Code. : " + RULE_LINK
 EX_IOERR = 74
 logger = logging.getLogger(constant.LOGGER_NAME)
+is_windows = platform.system() == 'Windows'
 
 
 class ResultItem:
@@ -115,15 +117,7 @@ def result_for_xml(result_item: ResultItem):
             items.set('msg', str_summary)
             _root_xml_item.append(items)
         else:
-            for file_name in (set(result_item._files_without_lic) | set(result_item._files_without_cop)):
-                items = ET.Element('error')
-                items.set('file', file_name)
-                items.set('id', 'rule_key_osc_checker_02')
-                items.set('line', '0')
-                items.set('msg', MSG_FOLLOW_LIC_TXT)
-                _root_xml_item.append(items)
-
-            for file_name in result_item._files_without_cop:
+            for file_name in (set(result_item._files_without_lic) | set(result_item._files_without_both)):
                 items = ET.Element('error')
                 items.set('file', file_name)
                 items.set('id', 'rule_key_osc_checker_02')
@@ -161,7 +155,7 @@ def write_result_html(result_file: str, exit_code: int, result_item: ResultItem,
         output_dir = os.path.dirname(result_file)
         Path(output_dir).mkdir(parents=True, exist_ok=True)
         html_result = result_for_html(result_item, project, path_to_find)
-        with open(result_file, 'w') as f:
+        with open(result_file, 'w', encoding='utf-8') as f:
             f.write(html_result)
         success = True
     except Exception as ex:
@@ -178,7 +172,7 @@ def write_result_yaml(result_file: str, exit_code: int, result_item: ResultItem)
         yaml_result = result_item.get_print_yaml()
         # Make yaml file
         with io.open(result_file, 'w', encoding='utf8') as outfile:
-            yaml.dump(yaml_result, outfile, default_flow_style=False, sort_keys=False)
+            yaml.dump(yaml_result, outfile, default_flow_style=False, sort_keys=False, allow_unicode=True)
         success = True
     except Exception as ex:
         logger.error(f"Error_to_write_yaml: {ex}")
@@ -216,7 +210,10 @@ def create_result_file(output_file_name, format='', _start_time=""):
 
 
 def get_path_in_yaml(oss_item):
-    return [os.path.join(oss_item.relative_path, file) for file in oss_item.source_name_or_path]
+    path_in_yaml = [os.path.join(oss_item.relative_path, file) for file in oss_item.source_name_or_path]
+    if is_windows:
+        path_in_yaml = [path.replace(os.sep, '/') for path in path_in_yaml]
+    return path_in_yaml
 
 
 def extract_files_in_path(remove_file_list, base_file_list, return_found=False):
@@ -242,8 +239,14 @@ def exclude_file_in_yaml(path_to_find, yaml_files, license_missing_files, copyri
     excluded_path = []
     lic_present_path = []
     cop_present_path = []
+    abnormal_yaml_files = {}
+
     for file in yaml_files:
-        oss_items, _ = parsing_yml(file, path_to_find)
+        oss_items, _, err_reason = parsing_yml(file, path_to_find, False)
+        # if oss_items is abnormal(empty or invalid)
+        if not oss_items:
+            abnormal_yaml_files[file] = err_reason
+
         for oss_item in oss_items:
             if oss_item.exclude:
                 excluded_path.extend(get_path_in_yaml(oss_item))
@@ -258,30 +261,67 @@ def exclude_file_in_yaml(path_to_find, yaml_files, license_missing_files, copyri
     license_missing_files = extract_files_in_path(lic_present_path, list(license_missing_files - set(files_with_exclude_removed)))
     copyright_missing_files = extract_files_in_path(cop_present_path, list(copyright_missing_files - set(files_with_exclude_removed)))
 
-    return license_missing_files, copyright_missing_files
+    return license_missing_files, copyright_missing_files, abnormal_yaml_files
+
+
+def get_total_file_list(path_to_find, prj_report, exclude_files):
+    if not path_to_find.endswith('/'):
+        path_to_find += '/'
+    total_files = [str(file_report.path).replace(path_to_find, '', 1) for file_report in prj_report.file_reports]
+    total_files_excluded = list(set(total_files) - set(exclude_files))
+    return total_files_excluded
+
+
+def add_reason_to_file_name(oss_pkg_info_files, abnormal_yaml_files, path_to_find):
+    if not path_to_find.endswith('/'):
+        path_to_find += '/'
+
+    for key, val in abnormal_yaml_files.items():
+        file_name = key.replace(path_to_find, '', 1)
+        if file_name in oss_pkg_info_files:
+            oss_pkg_info_files.remove(file_name)
+            if val == "yaml_error":
+                file_name += " (Yaml format error)"
+            elif val == "not_supported":
+                file_name += " (Not supported format)"
+            elif val == "empty":
+                file_name += " (Empty)"
+            else:
+                file_name += " (Unknown error)"
+            oss_pkg_info_files.append(file_name)
+    return oss_pkg_info_files
 
 
 def result_for_summary(path_to_find, oss_pkg_info_files, license_missing_files, copyright_missing_files,
-                       prj_report, _result_log, _check_only_file_mode, file_to_check_list, error_items):
+                       prj_report, _result_log, _check_only_file_mode, file_to_check_list, error_items, exclude_files):
     prechecker_compliant = False
     detected_lic = []
     missing_both_files = []
-    file_total = ""
+    file_total_num = ""
+    abnormal_yaml_files = {}
 
     if _check_only_file_mode:
-        file_total = len(file_to_check_list)
+        file_total_num = len(file_to_check_list)
     else:
-        file_total = len(prj_report.file_reports)
+        file_total_num = len(get_total_file_list(path_to_find, prj_report, exclude_files))
+
         # Get detected License
         for i, lic in enumerate(sorted(prj_report.used_licenses)):
-            detected_lic.append(lic)
+            if lic != '-':
+                detected_lic.append(lic)
 
     if oss_pkg_info_files:
         oss_yaml_files = find_sbom_yaml_files(path_to_find)
         # Exclude files in yaml
-        license_missing_files, copyright_missing_files = exclude_file_in_yaml(path_to_find, oss_yaml_files,
-                                                                              set(license_missing_files) - set(oss_pkg_info_files),
-                                                                              set(copyright_missing_files) - set(oss_pkg_info_files))
+        license_missing_files, copyright_missing_files, abnormal_yaml_files \
+            = exclude_file_in_yaml(path_to_find, oss_yaml_files,
+                                   set(license_missing_files) - set(oss_pkg_info_files),
+                                   set(copyright_missing_files) - set(oss_pkg_info_files))
+        # Subtract excluded files(untracked or ignored file)
+        oss_pkg_info_files = list(set(oss_pkg_info_files) - set(exclude_files))
+
+    # Add invalid format to file name
+    oss_pkg_info_files = add_reason_to_file_name(oss_pkg_info_files, abnormal_yaml_files, path_to_find)
 
     if len(license_missing_files) == 0 and len(copyright_missing_files) == 0:
         prechecker_compliant = True
@@ -296,7 +336,7 @@ def result_for_summary(path_to_find, oss_pkg_info_files, license_missing_files,
     result_item.compliant_result = prechecker_compliant
     result_item._oss_pkg_files = oss_pkg_info_files
     result_item._detected_licenses = detected_lic
-    result_item._count_total_files = file_total
+    result_item._count_total_files = file_total_num
     result_item._count_without_both = str(len(missing_both_files))
     result_item._count_without_lic = str(len(license_missing_files) + len(missing_both_files))
     result_item._count_without_cop = str(len(copyright_missing_files) + len(missing_both_files))
